<!DOCTYPE html>
<html ng-app="portroach">

    <head>
	<script type="text/javascript" src="assets/angular.min.js"></script>
    	<title>portroach - new distfile scanner</title>
	<link rel="icon" type="image/ico" href="assets/favicon.ico"/>
	<link href="assets/portroach.css" rel="stylesheet" type="text/css"/>
    </head>

    <body ng-controller="MaintainerController as maint" ng-init="maintainer='%%(maintainer)';predicate='basepkgpath'">
<div class="header">
  portroach - {{ maintainer }}'s ports
  <div class="menu">
    <a href="index.html">Home</a>
    <a href="restricted-ports.html">Restricted</a>
  </div>
</div>

    <hr>
    <p>
    	Notes:
    	<ul>
    		<li>Port version strings are derived from DISTNAME values; they are not in "ports format"</li>
    		<li>
    			Updated/Checked time is when the database was updated and when a new distfile scan was
    			done, respectively.
    		</li>
    	</ul>
    </p>

    <p>
    	<ul>
            <li>Port and New Version columns are clickable</li>
            <li>Table headings are clickable for different sorting.</li>
        </ul>
    </p>

    <p>
    	Key for the 'M' column:
    	<ul>
    		<li>G = file was found by sending incremental version guesses to server.</li>
    		<li>L = file was found in FTP or HTTP directory index or with dedicated handler</li>
    		<li>X = port is ignored - new versions are not reported.</li>
		<li>S = file was found with dedicated site handler.</li>
    	</ul>
    </p>

    <form class="filter-box">
      <fieldset class="filter-box">
        Filter Distfile (by pkgpath): <input tabindex="1" ng-model="search.basepkgpath">
        Only show outdated? <input tabindex="2" type="checkbox" ng-model="maint.onlyOutdated" ng-checked="maint.onlyOutdated">
      </fieldset>
    </form>

    <table class="maintainer-results results" cellspacing="1" cellpadding="2">
    	<tbody id="results">
    		<tr class="resultshead">
			<td><a href="" ng-click="predicate = 'basepkgpath'; reverse=false">Pkgpath</a></td>
    			<td><a href="" ng-click="predicate = 'ver'; reverse=false">Current</a></td>
    			<td><a href="" ng-click="predicate = 'newver'; reverse=false">New</a></td>
    			<td><a href="" ng-click="predicate = 'updated'; reverse=!reverse">Last Updated</a></td>
    			<td><a href="" ng-click="predicate = 'checked'; reverse=!reverse">Last Checked</a></td>
    			<td><a href="" ng-click="predicate = 'method'; reverse=false">M</a></td>
    		</tr>
	        <tr id="loading" ng-show="loading"><td colspan="7">Loading...</td></tr>
            <tr ng-cloak ng-class="maint.rowClass(port.newver, port.ignore)" ng-show="maint.showOutdated(port, maint.onlyOutdated)" ng-repeat="port in maint.ports | filter:search | orderBy:predicate:reverse">

                <td><div ng-bind-html="maint.parseHomepage(port.homepage, port.basepkgpath)"></div></td>
                <td><a href="http://cvsweb.openbsd.org/cgi-bin/cvsweb/ports/{{ port.basepkgpath }}">{{ port.ver }}</a></td>
                <td><a href="{{ port.newurl }}">{{ port.newver }}</a></td>
                <td>{{ port.updated }}</td>
                <td>{{ port.checked }}</td>
                <td>{{ port.method }}</td>
            </tr>
    	</tbody>
    </table>
    <script type="text/javascript" src="assets/app.js"></script>
</body>
</html>
