/*
 * Drop all portroach SQL tables.
 *
 * Copyright (C) 2006-2010, Shaun Amott <shaun@inerd.com>
 * All rights reserved.
 *
 */

DROP TABLE portdata;
DROP TABLE sitedata;
DROP TABLE portroach;
DROP TABLE stats;
DROP TABLE maildata;
DROP TABLE allocators;
DROP TABLE systemdata;
DROP TABLE results;
