<!DOCTYPE html>
<html ng-app="portroach">

<head>
	<script type="text/javascript" src="assets/angular.min.js"></script>
	<title>portroach - new distfile scanner</title>
	<link rel="icon" type="image/ico" href="assets/favicon.ico"/>
	<link href="assets/portroach.css" rel="stylesheet" type="text/css"/>
</head>

<body ng-controller="OverviewController as overview" ng-init="predicate='maintainer'">

<div class="header">
  portroach - OpenBSD Ports Distfile Scanner
  <div class="menu">
    <a href="index.html">Home</a>
    <a href="restricted-ports.html">Restricted</a>
  </div>
</div>

<hr>

<p class="intro">
  Generated on %%(date) at %%(time), by <a href="http://jasperla.github.io/portroach/">portroach</a> v%%(appver)
</p>
<div ng-cloak>
  <p class="summary">
    Summary:<br />
    Total distfiles: {{ overview.summary.total_ports }}<br />
    Outdated distfiles: {{ overview.summary.total_outdated }} ({{ overview.summary.outdated_percentage }}%)
  </p>
</div>

<form class="filter-box">
  <fieldset class="filter-box">
    Filter Maintainer: <input tabindex="1" ng-model="search.maintainer">
    Outdated only? <input tabindex="2" type="checkbox" ng-model="overview.onlyOutdated">
  </fieldset>
</form>

<table class="results" cellspacing="1" cellpadding="2" >
	<tbody id="results">
		<tr class="resultshead">
			<td><a class="sortable" title="Sort by maintainer" href="" ng-click="predicate = 'maintainer'; reverse=false">Maintainer</a></td>
			<td><a class="sortable" title="Sort by total distfiles" href="" ng-click="predicate = 'total'; reverse=!reverse">Total distfiles</a></td>
			<td><a class="sortable" title="Sort by distfiles (absolute)" href="" ng-click="predicate = 'withnewdistfile'; reverse=!reverse">New Distfile(s)</a></td>
			<td><a class="sortable" title="Sort by distfiles (relative)" href="" ng-click="predicate = 'percentage'; reverse=!reverse">% Out of Date</a></td>
		</tr>
		<tr ng-show="loading"><td colspan="4">Loading...</td></tr>
        <tr ng-cloak class="resultsrow" ng-show="overview.showOutdated(maintainer, overview.onlyOutdated)" ng-repeat="maintainer in overview.maintainers | filter:search | orderBy:predicate:reverse">
            <td><a href="{{ maintainer.maintainer }}.html">{{ overview.stripEmail(maintainer.maintainer) }}</a></td><td>{{ maintainer.total }}</td><td>{{ maintainer.withnewdistfile }}</td><td>{{ maintainer.percentage }}%</td>
        </tr>
	</tbody>
</table>
    <script type="text/javascript" src="assets/app.js"></script>
</body>
</html>
