body, table {
    font-family: "Verdana", "Helvetica", Sans-Serif;
}

h2 {
    font-family: "Verdana", "Helvetica", Sans-Serif;
}

tr {
    background-color: #FFFFFF;
    font-family: "Verdana", "Helvetica", Sans-Serif;
}

td, p, ul, span {
    font-family: "Verdana", "Helvetica", Sans-Serif;
    font-size: 85%;
}

.header {
    font-size:30px;
    font-weight:bold;
}

.intro {
    font-size:10px;
    color:#5c5c5c;
}

.menu {
    margin:4px;
    font-size:40%;
}

a {
    text-decoration:none;
}

.maintainer-results a {
    color: #000;
}

.summary {
}

.sortable {
    cursor:pointer;
}

.results {
    background-color: #444444;
    width: 80%;
}

.resultshead {
    color: #FFFFFF;
    font-weight: bold;
    background-color: #999999;
}

.resultsrow {
    background-color: #FFFFFF;
}

.resultsrowupdated {
    background-color: #AAAADD;
}

.resultshead a {
    color: #FFFFFF;
    text-decoration: none;
}

.box {
    border: 1px solid #000000;
    padding: 6px;
}

fieldset {
    display: block;
    border: 1px solid #000;
    padding-top:0.48em;
    padding-bottom: 0.625em;
    padding-left: 0.75em;
    padding-right: 0.75em;
    width:500px;
    margin-top: 20px;
    margin-bottom: 24px;
}

.filter-box {
    font-family: "Verdana", "Helvetica", Sans-Serif;
    font-size: 85%;
}
