<!DOCTYPE html>
<html ng-app="portroach">

    <head>
	<script type="text/javascript" src="assets/angular.min.js"></script>
        <title>portroach - new distfile scanner</title>
	<link rel="icon" type="image/ico" href="assets/favicon.ico"/>
	<link href="assets/portroach.css" rel="stylesheet" type="text/css"/>
    </head>

    <body ng-controller="RestrictedController as restricted" ng-init="predicate='basepkgpath'">

<div class="header">
    portroach - restricted ports
  <div class="menu">
    <a href="index.html">Home</a>
    <a href="restricted-ports.html">Restricted</a>
  </div>
</div>

<hr>

        <p>
            Notes:
            <ul>
                <li>
                    <b>Limit Regex</b> is a Perl regular expression used to limit the port's version.
                </li>
		        <li>
		            <b>Limit Even/Odd</b> tells portroach to limit the (<i>n</i>+1)th number to even or
	                odd numbers only.
	            </li>
	            <li>
	                Ports with a version number in their name are automatically dealt with.
	            </li>
	        </ul>
        </p>

    <form class="filter-box">
      <fieldset class="filter-box">
        Filter Distfile (by pkgpath): <input tabindex="1" ng-model="search.basepkgpath">
      </fieldset>
    </form>

    <table class="results" cellspacing="1" cellpadding="2">
      <tbody id="results">
            <tr class="resultshead">
                <td><a href="" ng-click="predicate = 'basepkgpath'; reverse=false">Pkgpath</a></td>
                <td>Limit Regex</td>
                <td>Limit Even/Odd</td>
                <td>Skip Versions</td>
		<td>Comment</td>
            </tr>

   	        <tr id="loading" ng-show="loading"><td colspan="5">Loading...</td></tr>

            <tr class="resultshow" ng-repeat="port in restricted.ports | filter:search | orderBy:predicate:reverse">
                <td><a href="http://cvsweb.openbsd.org/cgi-bin/cvsweb/ports/{{ port.basepkgpath }}">{{ port.basepkgpath }}</a></td>
                <td>{{ port.limitver }}</td>
                <td>{{ port.limitevenwhich }}</td>
                <td>{{ port.skipversions }}</td>
		<td>{{ port.pcfg_comment }}</td>
            </tr>
	  </tbody>
        </table>

    <script type="text/javascript" src="assets/app.js"></script>
</body>
</html>
